/*
  Copyright (C) 2001 Kimmo Pekkola

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/*
  $Header: //rainbox/cvsroot/Rainmeter/Plugin/MeterLine.cpp,v 1.1 2002/07/01 15:35:54 rainy Exp $

  $Log: MeterLine.cpp,v $
  Revision 1.1  2002/07/01 15:35:54  rainy
  Intial version


*/

#include "MeterLine.h"
#include "Measure.h"
#include "Error.h"
#include <crtdbg.h>

/*
** CMeterLine
**
** The constructor
**
*/
CMeterLine::CMeterLine() : CMeter()
{
	m_Autoscale = false;
	m_HorizontalLines = false;
	m_HorizontalPen = NULL;
	m_HorizontalColor = 0;
	m_CurrentPos = 0;
}

/*
** ~CMeterLine
**
** The destructor
**
*/
CMeterLine::~CMeterLine()
{
	DeleteObject(m_HorizontalPen);
	
	std::vector<HPEN>::iterator i = m_Pens.begin();
	for ( ; i != m_Pens.end(); i++)
	{
		DeleteObject(*i);
	}
}

/*
** Initialize
**
** create the pens for the lines
**
*/
void CMeterLine::Initialize(CMeterWindow& meterWindow)
{
	CMeter::Initialize(meterWindow);

	m_HorizontalPen = CreatePen(PS_SOLID, 0, m_HorizontalColor);

	std::vector<COLORREF>::iterator i = m_Colors.begin();
	for ( ; i != m_Colors.end(); i++)
	{
		m_Pens.push_back(CreatePen(PS_SOLID, 0, *i));
		m_AllValues.push_back(std::vector<unsigned int>());
	}
}

/*
** ReadConfig
**
** Read the meter-specific configs from the ini-file.
**
*/
void CMeterLine::ReadConfig(const char* filename, const char* section)
{
	char tmpSz[256];
	char tmpName[256];
	int i;

	// Read common configs
	CMeter::ReadConfig(filename, section);

	int lineCount = GetPrivateProfileInt( section, "LineCount", 1, filename);

	for (i = 0; i < lineCount; i++)
	{
		if (i == 0)
		{
			strcpy(tmpName, "LineColor");
		}
		else
		{
			sprintf(tmpName, "LineColor%i", i + 1);
		}

		if(GetPrivateProfileString(section, tmpName, "255, 255, 255", tmpSz, 255, filename) > 0) 
		{
			m_Colors.push_back(ParseColor(tmpSz));
		}

		if (i == 0)
		{
			strcpy(tmpName, "Scale");
		}
		else
		{
			sprintf(tmpName, "Scale%i", i + 1);
		}

		if(GetPrivateProfileString(section, tmpName, "1.0", tmpSz, 255, filename) > 0) 
		{
			m_ScaleValues.push_back(atof(tmpSz));
		}

		if (i != 0)
		{
			sprintf(tmpName, "MeasureName%i", i + 1);
			if(GetPrivateProfileString(section, tmpName, "", tmpSz, 255, filename) > 0) 
			{
				if (strlen(tmpSz) == 0)
				{
					std::string error = tmpSz;
					error += " is not defined in [";
					error += m_Name;
					error += "]";
					throw CError(error, __LINE__, __FILE__);
				}

				m_MeasureNames.push_back(tmpSz);
			}
		}
	}

	m_Autoscale = (1 == GetPrivateProfileInt( section, "AutoScale", 0, filename));

	m_HorizontalLines = (1 == GetPrivateProfileInt( section, "HorizontalLines", 0, filename));
	
	if(GetPrivateProfileString(section, "HorizontalLineColor", "0, 0, 0", tmpSz, 255, filename) > 0) 
	{
		m_HorizontalColor = ParseColor(tmpSz);
	}
}

/*
** Draw
**
** Draws the meter on the double buffer
**
*/
void CMeterLine::Draw(CMeterWindow& meterWindow)
{
	if (m_Measure == NULL) return;

	// Collect the values
	unsigned int value = m_Measure->GetValue();

	if (m_AllValues[0].size() < m_W)
	{
		m_AllValues[0].push_back(value);
	}
	else
	{
		m_AllValues[0][m_CurrentPos] = value;
	}

	int counter = 1;
	std::vector<CMeasure*>::iterator i = m_Measures.begin();
	for ( ; i != m_Measures.end(); i++)
	{
		value = (*i)->GetValue();

		if (m_AllValues[counter].size() < m_W)
		{
			m_AllValues[counter].push_back(value);
		}
		else
		{
			m_AllValues[counter][m_CurrentPos] = value;
		}
		counter++;
	}

	if (!IsHidden()) 
	{
		DrawColors(meterWindow);
	}

	m_CurrentPos++;
	if (m_CurrentPos >= m_W)
	{
		m_CurrentPos = 0;
	}
}

/*
** DrawColors
**
** Draws the meter as lines.
**
*/
void CMeterLine::DrawColors(CMeterWindow& meterWindow)
{
	unsigned int maxValue = 0;
	HDC bufferDC = meterWindow.GetDoubleBuffer();
	HPEN oldPen = (HPEN)SelectObject(bufferDC, m_Pens[0]);
	int counter = 0;

	// Find tha maximum value
	if (m_Autoscale)
	{
		unsigned int newValue = 0;
		std::vector< std::vector<unsigned int> >::iterator i = m_AllValues.begin();
		counter = 0;
		for (; i != m_AllValues.end(); i++)
		{
			std::vector<unsigned int>::iterator j = (*i).begin();
			for (; j != (*i).end(); j++)
			{
				newValue = max(newValue, (*j) * m_ScaleValues[counter]);
			}
			counter++;
		}

		// Scale the value up to nearest power of 2
		maxValue = 2;
		while(maxValue < newValue && maxValue != 0)
		{
			maxValue *= 2;
		}
	}
	else
	{
		maxValue = m_Measure->GetMaxValue();

		std::vector<CMeasure*>::iterator i = m_Measures.begin();
		for (; i != m_Measures.end(); i++)
		{
			maxValue = max(maxValue, (*i)->GetMaxValue());
		}
	}

	// Draw the horizontal lines
	if (m_HorizontalLines)
	{
		// Calc the max number of lines we should draw
		int maxLines = m_H / 4;	// one line per 4 pixels is max
		int numOfLines;

		// Check the highest power of 2 that fits in maxLines
		int power = 2;
		while(power < maxLines)
		{
			power *= 2;
		}

		numOfLines = (maxValue % power) + 1;

		SelectObject(bufferDC, m_HorizontalPen);

		int Y;
		for (int j = 0; j < numOfLines; j++)
		{
			Y = (j + 1) * m_H / (numOfLines + 1);
			Y = m_Y + m_H - Y - 1;
			MoveToEx(bufferDC, 0, Y, NULL);
			LineTo(bufferDC, m_W + 1, Y);
		}
	}

	// Draw all the lines
	counter = 0;
	std::vector< std::vector<unsigned int> >::iterator i = m_AllValues.begin();
	for (; i != m_AllValues.end(); i++)
	{
		// Draw a line
		int X = m_X;
		int Y = 0;
		int pos = m_CurrentPos + 1;
		if (pos >= m_W) pos = 0;
		
		SelectObject(bufferDC, m_Pens[counter]);

		for (int j = 0; j < m_W; j++)
		{
			if (pos < (*i).size())
			{
				Y = (*i)[pos] * m_ScaleValues[counter] * (m_H - 1) / (double)maxValue;
			}
			else
			{
				Y = 0;			
			}

			Y = m_Y + m_H - Y - 1;

			if (j == 0)
			{
				MoveToEx(bufferDC, X, Y, NULL);
			}
			else
			{
				LineTo(bufferDC, X, Y);
			}

			X++;
			pos++;
			if (pos >= m_W) pos = 0;
		}
		LineTo(bufferDC, X, Y);	// Draw the last pixel

		counter++;
	}

	SelectObject(bufferDC, oldPen);
}

/*
** BindMeasure
**
** Overwritten method to handle the other measure bindings.
**
*/
void CMeterLine::BindMeasure(std::list<CMeasure*>& measures)
{
	CMeter::BindMeasure(measures);

	std::vector<std::string>::iterator j = m_MeasureNames.begin();
	for (; j != m_MeasureNames.end(); j++)
	{
		// Go through the list and check it there is a secondary measure for us
		std::list<CMeasure*>::iterator i = measures.begin();
		for( ; i != measures.end(); i++)
		{
			if(_stricmp((*i)->GetName(), (*j).c_str()) == 0)
			{
				m_Measures.push_back(*i);
				break;
			}
		}
		
		if (i == measures.end())
		{
	        throw CError(std::string("The meter [") + m_Name + "] cannot be bound with [" + (*j) + "]!", __LINE__, __FILE__);
		}
	}
}
